// (Heavily) adapted from https://github.com/G6EJD/ESP32-8266-Audio-Spectrum-Display/blob/master/ESP32_Spectrum_Display_02.ino
// Adjusted to allow brightness changes on press+hold, Auto-cycle for 3 button presses within 2 seconds
// Edited to add Neomatrix support for easier compatibility with different layouts.

// ESP32
#include <FastLED_NeoMatrix.h>
#include <arduinoFFT.h>
#include <EasyButton.h>
#include <EEPROM.h>


#define SAMPLES         1024          // Must be a power of 2
#define SAMPLING_FREQ   40000         // Hz, must be 40000 or less due to ADC conversion time. Determines maximum frequency that can be analysed by the FFT Fmax=sampleF/2.

#define AUDIO_IN_PIN    36           // Signal in on this pin
#define LED_PIN         5             // LED strip data
#define BTN_PIN         23             // Connect a push button to this pin to change patterns
#define LONG_PRESS_MS   200           // Number of ms to count as a long press
#define COLOR_ORDER     GRB           // If colours look wrong, play with this
#define CHIPSET         WS2812B       // LED strip type
//#define MAX_MILLIAMPS   10000          // Careful with the amount of power here if running off USB port
const int BRIGHTNESS_SETTINGS[3] = {10, 50, 250};  //Tablica 3 liczb całkowitych dla 3 ustawień jasności (na podstawie wciśnięcia+przytrzymania BTN_PIN)
#define LED_VOLTS       5             // Usually 5 or 12
#define NUM_BANDS       20          // Aby to zmienić, będziesz musiał zmienić kilka instrukcji if opisujących mapowanie z binów na pasma 
const uint8_t kMatrixWidth = 40;                          // Matrix width
const uint8_t kMatrixHeight = 16;                         // Matrix height
#define NUM_LEDS       (kMatrixWidth * kMatrixHeight)     // Total number of LEDs
#define BAR_WIDTH      (kMatrixWidth  / (NUM_BANDS - 1))  // Jeśli szerokość >= 8 świeci 1 szerokość diody LED na pasek, >= 16 świeci 2 diody LED szerokość paska itp
#define TOP            (kMatrixHeight - 0)                // Don't allow the bars to go offscreen
#include "zmienne.h"





//arduinoFFT FFT = arduinoFFT(vReal, vImag, SAMPLES, SAMPLING_FREQ);
arduinoFFT FFT = arduinoFFT();
TaskHandle_t Task1;

void IRAM_ATTR onTimer() {

  if ( sample_gotowe == 0) {
    bufor_sampli[licznik_sampli] = analogRead(AUDIO_IN_PIN);
    licznik_sampli++;
    if (licznik_sampli >= SAMPLES) {
      sample_gotowe = 1;
      licznik_sampli = 0;
    }
  }
}


void setup() { // =========================================

  Serial.begin(115200);
  Serial.println("start widmo");
  // analogSetAttenuation(ADC_11db); //ADC_0db, ADC_2_5db, ADC_6db, ADC_11db

  pinMode(18, OUTPUT);
  pinMode(19, OUTPUT);

  xTaskCreatePinnedToCore(
    Task1code,   /* Task function. */
    "Task1",     /* name of task. */
    10000,       /* Stack size of task */
    NULL,        /* parameter of the task */
    1,           /* priority of the task */
    &Task1,      /* Task handle to keep track of created task */
    0);          /* pin task to core 0 */
  delay(500);

  disableCore0WDT();
  disableCore1WDT();

  FastLED.addLeds<CHIPSET, LED_PIN, COLOR_ORDER>(leds, NUM_LEDS).setCorrection(TypicalSMD5050);
  //FastLED.setMaxPowerInVoltsAndMilliamps(LED_VOLTS, MAX_MILLIAMPS);
  FastLED.setBrightness(BRIGHTNESS_SETTINGS[1]);
  FastLED.clear();

  modeBtn.begin();
  modeBtn.onPressed(changeMode);
  modeBtn.onPressedFor(LONG_PRESS_MS, brightnessButton);
  modeBtn.onSequence(3, 2000, startAutoMode);
  modeBtn.onSequence(5, 2000, brightnessOff);
}

void changeMode() {
  Serial.println("Button pressed");
  if (FastLED.getBrightness() == 0) FastLED.setBrightness(BRIGHTNESS_SETTINGS[0]);  //Re-enable if lights are "off"
  autoChangePatterns = false;
  buttonPushCounter = (buttonPushCounter + 1) % 9;
}

void startAutoMode() {
  autoChangePatterns = true;
}

void brightnessButton() {
  if (FastLED.getBrightness() == BRIGHTNESS_SETTINGS[2])  FastLED.setBrightness(BRIGHTNESS_SETTINGS[0]);
  else if (FastLED.getBrightness() == BRIGHTNESS_SETTINGS[0]) FastLED.setBrightness(BRIGHTNESS_SETTINGS[1]);
  else if (FastLED.getBrightness() == BRIGHTNESS_SETTINGS[1]) FastLED.setBrightness(BRIGHTNESS_SETTINGS[2]);
  else if (FastLED.getBrightness() == 0) FastLED.setBrightness(BRIGHTNESS_SETTINGS[0]); //Re-enable if lights are "off"
}

void brightnessOff() {
  FastLED.setBrightness(0);  //Lights out
}

//=========================================================== LOOP Core 1

void loop() {


  digitalWrite(18, !digitalRead(18)); // do oscyloskopu FPS

  FastLED.clear();

  modeBtn.read();

  Zerowanie_slupkow();
  Przepisanie_sampli(); 
  Oblicz_FFT();
  Przetwarzaj_slupki();

}


// Task1code: Core 0  =============================================== CORE 0
void Task1code( void * pvParameters ) {
  Serial.print("Task2 running on core ");
  Serial.println(xPortGetCoreID());
  My_timer = timerBegin(0, 80, true);
  timerAttachInterrupt(My_timer, &onTimer, true);
  timerAlarmWrite(My_timer, 25, true ); //40kHz
  timerAlarmEnable(My_timer);

  for (;;) {

    Odbior_UART();
    Jasnosc();
    Efekty();
    Piki();
    dane_uart = "";

  }
}


// PATTERNS BELOW //

void rainbowBars(int band, int barHeight) {
  int xStart = BAR_WIDTH * band;
  for (int x = xStart; x < xStart + BAR_WIDTH; x++) {
    for (int y = TOP - 1; y >= (TOP - 1) - barHeight; y--) {
      matrix->drawPixel(x, y, CHSV((x / BAR_WIDTH) * (255 / NUM_BANDS), 255, 255));
    }
  }
}



void kolorBars(int band, int barHeight) {
  int xStart = BAR_WIDTH * band;
  for (int x = xStart; x < xStart + BAR_WIDTH; x++) {
    for (int y = TOP; y >= TOP - barHeight; y--) {
      matrix->drawPixel(x, y, ColorFromPalette(greenbluePal, y * (255 / (barHeight + 1))));
    }
  }
}

void greenBars(int band, int barHeight) {
  int xStart = BAR_WIDTH * band;
  for (int x = xStart; x < xStart + BAR_WIDTH; x++) {
    for (int y = TOP - 1; y >= (TOP - 1) - barHeight; y--) {
      matrix->drawPixel(x, y , CHSV(72, 240, 255));
    }
  }
}

void redBars(int band, int barHeight) {
  int xStart = BAR_WIDTH * band;
  for (int x = xStart; x < xStart + BAR_WIDTH; x++) {
    for (int y = TOP - 1; y >= (TOP - 1) - barHeight; y--) {
      matrix->drawPixel(x, y , CHSV(0, 240, 255));
    }
  }
}

void blueBars(int band, int barHeight) {
  int xStart = BAR_WIDTH * band;
  for (int x = xStart; x < xStart + BAR_WIDTH; x++) {
    for (int y = TOP - 1; y >= (TOP - 1) - barHeight; y--) {
      matrix->drawPixel(x, y , CHSV(124, 240, 255));
    }
  }
}

void yellowBars(int band, int barHeight) {
  int xStart = BAR_WIDTH * band;
  for (int x = xStart; x < xStart + BAR_WIDTH; x++) {
    for (int y = TOP - 1; y >= (TOP - 1) - barHeight; y--) {
      matrix->drawPixel(x, y , CHSV(38, 240, 255));
    }
  }
}

void purpleBars(int band, int barHeight) {
  int xStart = BAR_WIDTH * band;
  for (int x = xStart; x < xStart + BAR_WIDTH; x++) {
    for (int y = TOP; y >= TOP - barHeight; y--) {
      matrix->drawPixel(x, y, ColorFromPalette(purplePal, y * (255 / (barHeight + 1))));
    }
  }
}

void changingBars(int band, int barHeight) {
  int xStart = BAR_WIDTH * band;
  for (int x = xStart; x < xStart + BAR_WIDTH; x++) {
    for (int y = TOP - 1; y >= (TOP - 1) - barHeight; y--) {
      matrix->drawPixel(x, y, CHSV(y * (255 / kMatrixHeight) + colorTimer, 255, 255));
    }
  }
}

void greenredBars(int band, int barHeight) {
  int xStart = BAR_WIDTH * band;
  for (int x = xStart; x < xStart + BAR_WIDTH; x++) {
    for (int y = TOP - 1; y >= (TOP - 1) - barHeight; y--) {
      matrix->drawPixel(x, y, CHSV(y * (120 / kMatrixHeight) + 239, 255, 255));
    }
  }
}

void centerBars(int band, int barHeight) {
  int xStart = BAR_WIDTH * band;
  for (int x = xStart; x < xStart + BAR_WIDTH; x++) {
    if (barHeight % 2 == 0) barHeight--;
    int yStart = ((kMatrixHeight - barHeight) / 2 );
    for (int y = yStart; y <= (yStart + barHeight); y++) {
      int colorIndex = constrain((y - yStart) * (255 / barHeight), 0, 255);
      matrix->drawPixel(x, y, ColorFromPalette(heatPal, colorIndex));
    }
  }
}

void whitePeak(int band) {
  int xStart = BAR_WIDTH * band;
  int peakHeight = TOP - peak[band] - 1;
  for (int x = xStart; x < xStart + BAR_WIDTH; x++) {
   //  matrix->drawPixel(x, peakHeight, CHSV(43, 105, 255));
  // matrix->drawPixel(x, peakHeight, CHSV(60, 255, 255));
   //matrix->drawPixel(x, peakHeight, CHSV(43, 180, 255));// kremowy
matrix->drawPixel(x, peakHeight, CHSV(53, 175, 239)); // kolor peek
  }
}


void outrunPeak(int band) {
  int xStart = BAR_WIDTH * band;
  int peakHeight = TOP - peak[band] - 1;
  for (int x = xStart; x < xStart + BAR_WIDTH; x++) {
    matrix->drawPixel(x, peakHeight, ColorFromPalette(outrunPal, peakHeight * (255 / kMatrixHeight)));
  }
}


void Oblicz_FFT() {

  // Compute FFT
  // FFT.DCRemoval();
  // FFT.Windowing(FFT_WIN_TYP_HAMMING, FFT_FORWARD);
  // FFT.Compute(FFT_FORWARD);
  // FFT.ComplexToMagnitude();

  FFT.DCRemoval();
  FFT.Windowing(vReal, SAMPLES, FFT_WIN_TYP_HAMMING, FFT_FORWARD);
  FFT.Compute(vReal, vImag, SAMPLES, FFT_FORWARD);
  FFT.ComplexToMagnitude(vReal, vImag, SAMPLES);
  FFT.MajorPeak(vReal, SAMPLES, SAMPLING_FREQ);



  // Analyse FFT results
  for (int i = 2; i < (SAMPLES / 2); i++) {    // Don't use sample 0 and only first SAMPLES/2 are usable. Each array element represents a frequency bin and its value the amplituda.
    if (vReal[i] > szumy) {

      // 20 bands, 10kHz top band
      if (i <= 2 )           bandValues[0]  += (int)vReal[i];
      if (i > 2   && i <= 3  ) bandValues[1]  += (int)vReal[i];
      if (i > 3  && i <= 5  ) bandValues[2]  += (int)vReal[i];
      if (i > 5  && i <= 8  ) bandValues[3]  += (int)vReal[i];
      if (i > 8   && i <= 10  ) bandValues[4]  += (int)vReal[i];
      if (i > 10   && i <= 12  ) bandValues[5]  += (int)vReal[i];
      if (i > 12   && i <= 16 ) bandValues[6]  += (int)vReal[i];
      if (i > 16   && i <= 20  ) bandValues[7]  += (int)vReal[i];
      if (i > 20   && i <= 25  ) bandValues[8]  += (int)vReal[i];
      if (i > 25   && i <= 30  ) bandValues[9]  += (int)vReal[i];
      if (i > 30   && i <= 36  ) bandValues[10]  += (int)vReal[i];
      if (i > 36   && i <= 45  ) bandValues[11]  += (int)vReal[i];
      if (i > 45   && i <= 55  ) bandValues[12]  += (int)vReal[i];
      if (i > 55   && i <= 66  ) bandValues[13]  += (int)vReal[i];
      if (i > 66   && i <= 82  ) bandValues[14]  += (int)vReal[i];
      if (i > 82   && i <= 106  ) bandValues[15]  += (int)vReal[i];
      if (i > 106   && i <= 137  ) bandValues[16]  += (int)vReal[i];
      if (i > 137   && i <= 176  ) bandValues[17]  += (int)vReal[i];
      if (i > 176   && i <= 227  ) bandValues[18]  += (int)vReal[i];
      if (i > 227             ) bandValues[19]  += (int)vReal[i];


    }
  }
}

void Przetwarzaj_slupki() {

  // Przetwarzaj dane FFT na wysokosc słupków
  for (byte band = 0; band < NUM_BANDS; band++) {

    // Skaluj paski wyświetlacza
    int barHeight = bandValues[band] / amplituda;

    //ARW
    if (barHeight > TOP && arw_on == true) {
      amplituda = amplituda + arw_spadek;
    }

    if (barHeight > TOP) barHeight = TOP;

    // Mała wielkosc uśredniania między klatkami
    barHeight = ((oldBarHeights[band] * 1) + barHeight) / 2;

    // Przenieś peak w górę
    if (barHeight > peak[band]) {
      peak[band] = min(TOP, barHeight);
    }

    // rysuj słupki
    switch (buttonPushCounter) {
      case 0:
        rainbowBars(band, barHeight);
        break;
      case 1:
        yellowBars(band, barHeight);
        break;
      case 2:
        greenBars(band, barHeight);
        break;
      case 3:
        redBars(band, barHeight);
        break;
      case 4:
        blueBars(band, barHeight);
        break;
      case 5:
        changingBars(band, barHeight);
        break;
      case 6:
        greenredBars(band, barHeight);
        break;
      case 7:
        whitePeak(band);
        break;
      case 8:
        purpleBars(band, barHeight);
        break;
    }

    // rysuj  piki

    if (pik == true)  whitePeak(band);
     
    // Zapisz oldBarHeights do późniejszego uśredniania
    oldBarHeights[band] = barHeight;
  }

  // Decay peak
  EVERY_N_MILLISECONDS(60) {
    for (byte band = 0; band < NUM_BANDS; band++)
      if (peak[band] > 0) peak[band] -= 1;
    colorTimer++;
  }

  // Used in some of the patterns
  EVERY_N_MILLISECONDS(10) {
    colorTimer++;
  }

  EVERY_N_SECONDS(10) {
    if (autoChangePatterns) buttonPushCounter = (buttonPushCounter + 1) % 9;
  }

  FastLED.show();

  if (arw_on == true) {
    amplituda = amplituda - arw_wzrost;
    if (amplituda <= prog_arw) amplituda = prog_arw;
  }
}

void Zerowanie_slupkow() {

  for (int i = 0; i < NUM_BANDS; i++) {
    bandValues[i] = 0;
  }

}

void Przepisanie_sampli() {
  for (word x = 0; x < SAMPLES ; x++) {

    vReal[x] = bufor_sampli[x];
    vImag[x] = 0;

  }
  sample_gotowe = 0;
}

void Odbior_UART() {  // bufor odbior UART

  if (Serial.available() > 0) {

    dane_uart = Serial.readStringUntil('\n');
  }
}


void Jasnosc() {

  if (dane_uart.substring(0, 4) == "jas+") { // odbiór UART z Atmegi
    jasnosc++;  //
    if (jasnosc > 25) jasnosc = 25; 
    FastLED.setBrightness(krzywa[jasnosc]); 
  }
  else if (dane_uart.substring(0, 4) == "jas-") {
    jasnosc--; 
    if ( jasnosc <= 1) jasnosc = 1;
    FastLED.setBrightness(krzywa[jasnosc]);
  }
}

void Efekty() {

  if (dane_uart.substring(0, 3) == "ef+") {
    buttonPushCounter++;
    if (buttonPushCounter > 8) buttonPushCounter = 0;
  }
  else if (dane_uart.substring(0, 3) == "ef-") {
 buttonPushCounter--;
    if (buttonPushCounter < 0) buttonPushCounter = 8;
  }
}


  void Piki() {
    if (dane_uart.substring(0, 4) == "peak") {
      pik = !pik;
    }
  }



 