/*
  Arduino IDE 2.0.3, Pakiet MiniCore / Atmega88PA, external 8MHz 
 
*/
// PORTC2 LED programowania odbiornika IR
// PORTC4 LED czuwania
// PORTC3 LED rezerwa

// Przycisk zasilania PIND4
// Przycisk programowania odbiornika IR PIND2

#include <avr/io.h>
#include <avr/interrupt.h>
#include <EEPROM.h>
#define PRZYCISK !bitRead(PIND,4)
#define CZUJNIK_IR !bitRead(PIND,3)
#define ZASILANIE !bitRead(PIND,2)

volatile uint8_t licznik_0, licznik_1, zapis;
uint8_t kod;
volatile bool flaga_stanu = true, porownanie = true;
volatile bool zapis_rozkaz = 0;
volatile uint16_t  adres , rozkaz;
volatile uint8_t tab[71], dlugosc, x;

int main (void) {

  Serial.begin(115200); Serial.println("start programu ");
  DDRD = B11110011; PORTD = B11111111;
  DDRC = B11111111; PORTC = B10001110;
  DDRB = B11111111; PORTB = B11111111;

  cli(); // zatrzymanie przerwan

  // ========== timer2 ==================
  TCCR2B = 0;
  TIMSK2 = 0;
  bitSet(TCCR2B, CS20);   //preskaler 1
  bitClear(TIMSK2, TOIE2);   // wylacz przerwania timera2

  sei(); // wlaczenie przerwan

  while (1) {

    if (ZASILANIE) { // przycisk zasilania

      bitWrite(PORTC, 5, !bitRead(PORTC, 5)); // przekaznik zasilania
      bitWrite(PORTC, 4, bitRead(PORTC, 5));  // LED czuwania
      bitWrite(PORTB, 0, !bitRead(PORTC, 5)); // przekaznik zasilania
      _delay_ms(1000);
    }

    if (CZUJNIK_IR) {    // jesli wyjscie IR = 0
      bitSet(TIMSK2, TOIE2);   // wlacz przerwanie timera2
    }

    if (zapis == true && rozkaz > 0) { 

      if (rozkaz == 1)Serial.println("1");
      if (rozkaz == 71) Serial.println("2");
      if (rozkaz == 141) Serial.println("3");
      if (rozkaz == 211) Serial.println("4");
      if (rozkaz == 281) Serial.println("5");
      if (rozkaz == 351) Serial.println("6");

      bitSet(PORTC, 2); // LED programowania
      _delay_ms(300);
      bitClear(PORTC, 2); // LED programowania
      rozkaz = 0;
    }
    else {
   // if (zapis == false) {
      if (rozkaz > 0) { // wykonanie rozkazów

        if (rozkaz == 1) {
          bitWrite(PORTC, 5, !bitRead(PORTC, 5)); // przekaznik zasilania
         bitWrite(PORTB, 0, !bitRead(PORTC, 5)); // przekaznik zasilania
          bitWrite(PORTC, 4, bitRead(PORTC, 5));  // LED czuwania
           _delay_ms(1000);
        }
        if (rozkaz == 71) Serial.println("jas+");
        if (rozkaz == 141) Serial.println("jas-");
        if (rozkaz == 211) Serial.println("ef+");
        if (rozkaz == 281)  Serial.println("ef-");
        if (rozkaz == 351) Serial.println("peak");

        _delay_ms(200);
        rozkaz = 0;
      }
    }

    if (zapis_rozkaz == true) { //flaga zapisu rozkazu

      EEPROM.put(adres, tab); //zapis tablicy rozkazu z pilota
      zapis_rozkaz = false;
      rozkaz = adres;
      adres += 70; // nastepny adres do zapisu
     
      if (adres > 351) {

        bitSet(PORTC, 2); // wyłącz LED programowania
        bitWrite(PORTC, 4, bitRead(PORTC, 5));  // LED czuwania
        Serial.println("koniec");
        adres = 1 ;
        zapis = false;
      }
    }

    if (PRZYCISK) {
      Programowanie();
    }
  }
}


ISR(TIMER2_OVF_vect) { // 32us

bitWrite(PORTC, 1, !bitRead(PORTC, 1)); 

  if (CZUJNIK_IR) {          // czytanie pilota - zera
    if (flaga_stanu == true) {
      licznik_0 = 0 ;
      tab[x] = licznik_1 / 8;
      x++;
    }
    licznik_0 ++;
    flaga_stanu = false;
  }
  else {
    if (flaga_stanu == false) {   // czytanie pilota - jedynki
      licznik_1 = 0;
      tab[x] = licznik_0 / 8;
      x++;
    }
    licznik_1 ++ ;
    flaga_stanu = true;
  }

  if (licznik_1 == 255 || x >= 70) {
    dlugosc = x;
    tab[dlugosc] = x;

    // Zapis danych z pilota
    if (zapis == true && dlugosc > 20)  zapis_rozkaz = true;

    // Odczyt danych z pilota
    if (zapis == false && dlugosc > 20 )   {

      // Porownanie danych z pilota z EEPROM
      for (adres = 1; adres <= 351; adres += 70) {
        porownanie = true;

        for (uint16_t i = 0; i <= dlugosc; i++) {

          EEPROM.get(i + adres, kod);
          kod = kod - 1 ; // korekta błedu 
          if (kod != tab[i]) {
            kod ++ ;
            if (kod != tab[i]) {
              kod ++ ;
              if (kod != tab[i]) {
                porownanie = false;  // nie znaleziono kodu
                break;
              }
            }
          }
        }
        if (porownanie == true) {  // kod odnaleziony
          rozkaz = adres;
          break;
        }
      }
    }

    x = 0 ;
    dlugosc = 0;
    licznik_1 = 0;
    bitClear(TIMSK2, TOIE2);   //wyłacz timer2
  }
}


void Programowanie() {   // programowanie odbiornika IR

  if (!zapis) {
    Serial.println("programowanie");
    bitClear(PORTC, 2); //Włacz LED programowania
    bitSet(PORTC, 4); // wylacz LED czuwania
    adres = 1;
    zapis = true;
  }
}
